@@ -20,6 +20,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "osipuacb.h"
 #include "linphonecore.h"
 #include "misc.h"
+#include <mediastream.h>
+#include <mssync.h>
 
 /* these are useful messages for the status */
 extern gchar *ready;
@@ -60,11 +62,13 @@ int notifytransfer_func_cb(OsipDialog *dialog, transaction_t *trn, sip_t *msg, v
 
     OsipUA *ua=osip_dialog_get_ua(dialog);
     LinphoneCore *lc=(LinphoneCore*)ua->data;
+    char *callee;
     
     if (lc->refercall == NULL) 
 	return  0;
     
-    err=osip_dialog_notify(lc->refercall);
+    err=osip_dialog_notify(lc->refercall, NULL, 
+			   /* event package */ NULL, NULL, NULL);
     if (err!=0){
 	g_warning("error in notify: %s (%i)\n",strerror(err),err);
 	return -1;
@@ -159,7 +163,7 @@ int bye_cb(OsipDialog *dialog, transaction_t *trn, sip_t *msg, void *p2)
 	return 0;
 }
 
-int faillure_cb(OsipDialog *dialog, transaction_t *trn, sip_t *msg, void *p2)
+int failure_cb(OsipDialog *dialog, transaction_t *trn, sip_t *msg, void *p2)
 {
 	int err=((int*)p2)[0];
 	char *reason;
@@ -220,7 +224,7 @@ int faillure_cb(OsipDialog *dialog, transaction_t *trn, sip_t *msg, void *p2)
 				lc->vtable.display_status(lc,reason);
 				sfree(reason);
 			}
-			else g_message("faillure_cb: %s\n",strerror(-err));	
+			else g_message("failure_cb: %s\n",strerror(-err));	
 	}
 	if (lc->ringstream!=NULL) {
 		ring_stop(lc->ringstream);
@@ -232,6 +236,8 @@ int faillure_cb(OsipDialog *dialog, transaction_t *trn, sip_t *msg, void *p2)
 	    audio_stream_stop(lc->audiostream);
 	    lc->audiostream=NULL;
 	}
+	if (lc->vtable.call_failure)
+	  lc->vtable.call_failure(lc, err);
 	if (diaparams!=NULL) linphone_dialog_params_destroy(diaparams);
 	lc->call=NULL;
 	linphone_core_thread_leave(lc);	
@@ -348,7 +354,7 @@ int set_audio_offer(SdpHandler *sdph,SdpContext *ctx)
 {
 	LinphoneCore *lc;
 	OsipUA *ua;
-	MSCodecInfo *codec;
+	PayloadType *codec;
 	GList *elem;
 	SdpPayload payload;
 	ua=body_handler_get_ua(BODY_HANDLER(sdph));
@@ -361,28 +367,29 @@ int set_audio_offer(SdpHandler *sdph,SdpContext *ctx)
 		sdp->c_connection->c_addr=sgetcopy(lc->net_conf.nat_address);
 	}
 	
-	/*REVISIT: we should use local profile to setup the offer instead of the 
-		mediastreamer codec list*/
+	
 	elem=lc->codecs_conf.audio_codecs;
 	while(elem!=NULL){
-		codec=(MSCodecInfo*) elem->data;
-		if (codec->is_usable && codec->is_selected){
+		codec=(PayloadType*) elem->data;
+		if (payload_type_usable(codec) && payload_type_enabled(codec)){
 			
 			sdp_payload_init(&payload);
-#if 1
-			payload.a_rtpmap=g_strdup_printf("%s/8000/1",codec->description);
-#else
-			payload.a_rtpmap=g_strdup_printf("%s/8000",codec->description);
-#endif
+			payload.a_rtpmap=g_strdup_printf("%s/%i/1",codec->mime_type,codec->clock_rate);
 			payload.pt=rtp_profile_get_payload_number_from_rtpmap(lc->local_profile,payload.a_rtpmap);
 			payload.localport=lc->rtp_conf.audio_rtp_port;
 			/* when setting speex, add a b=<AS>: to choose bitrate */
-			if (strcmp(codec->description,"speex")==0){
-				int ctype=linphone_core_get_connection_type(lc);
-				if (ctype<CONNECTION_TYPE_NUMERIS)
-					payload.b_as_bandwidth=8;
-				else payload.b_as_bandwidth=20;
+			if (strcmp(codec->mime_type,"speex")==0
+			    || strcmp(codec->mime_type,"iLBC")==0){
+				payload.b_as_bandwidth=codec->normal_bitrate/1000;
 			}
+			if (strcmp(codec->mime_type,"iLBC")==0){
+			  /* set mode to choose 20ms or 30ms frame version */
+			  if (codec->normal_bitrate == 15200)
+			    payload.a_fmtp = "mode=20";
+			  else
+			    payload.a_fmtp = "mode=30";
+			}
+
 			sdp_handler_add_audio_payload(sdph,ctx,&payload);
 			g_free(payload.a_rtpmap);
 		}
@@ -402,20 +409,18 @@ int set_video_offer(SdpHandler *sdph,SdpContext *ctx)
 {
 	LinphoneCore *lc;
 	OsipUA *ua;
-	MSCodecInfo *codec;
+	PayloadType *codec;
 	GList *elem;
 	ua=body_handler_get_ua(BODY_HANDLER(sdph));
 	lc=(LinphoneCore*)ua->data;
 	elem=lc->codecs_conf.video_codecs;
 	while(elem!=NULL){
-		codec=(MSCodecInfo*) elem->data;
-		if (codec->is_usable && codec->is_selected){
+		codec=(PayloadType*) elem->data;
+		if (payload_type_usable(codec) && payload_type_enabled(codec)){
 			SdpPayload payload;
 			PayloadType *rtpp;
 			sdp_payload_init(&payload);
-			payload.pt=rtp_profile_get_payload_number_from_mime(lc->local_profile,codec->description);
-			rtpp=rtp_profile_get_payload(lc->local_profile,payload.pt);
-			payload.a_rtpmap=g_strdup_printf("%s/%i",rtpp->mime_type,rtpp->clock_rate);
+			payload.a_rtpmap=g_strdup_printf("%s/%i",codec->mime_type,codec->clock_rate);
 			payload.localport=lc->rtp_conf.video_rtp_port;
 			sdp_handler_add_video_payload(sdph,ctx,&payload);
 			g_free(payload.a_rtpmap);
@@ -438,17 +443,23 @@ int payload_is_supported(SdpPayload *payload,RtpProfile *local_profile,RtpProfil
 	if (localpt>=0){
 		/* this payload is understood, but does the user want to use it ?? */
 		PayloadType *rtppayload;
-		MSCodecInfo *codec;
 		rtppayload=rtp_profile_get_payload(local_profile,localpt);
-		if (rtppayload==NULL) return 0;
-		codec=payload_type_get_user_data(rtppayload);
-		if (codec==NULL) return 0;
-		if (!(codec->is_usable && codec->is_selected)) return 0;
+		if (rtppayload==NULL) {
+			g_warning("strange error !!");
+			return 0;
+		}
+		if (strcmp(rtppayload->mime_type,"telephone-event")!=0){
+			if (!(payload_type_usable(rtppayload) && payload_type_enabled(rtppayload))) {
+				g_warning("payload %s is not usable or enabled.",rtppayload->mime_type);
+				return 0;
+			}
+		}
 		/* this payload is supported in our local rtp profile, so add it to the dialog rtp
 		profile */
-		rtp_profile_set_payload(dialog_profile,payload->pt,payload_type_clone(rtppayload));
+		rtppayload=payload_type_clone(rtppayload);
+		rtp_profile_set_payload(dialog_profile,payload->pt,rtppayload);
 		/* add to the rtp payload type some other parameters (bandwidth) */
-		if (payload->b_as_bandwidth!=0) rtppayload->normal_bitrate=payload->b_as_bandwidth;
+		if (payload->b_as_bandwidth!=0) rtppayload->normal_bitrate=payload->b_as_bandwidth*1000;
 		return 1;
 	}
 	return 0;
@@ -514,7 +525,7 @@ int accept_video_offer(SdpHandler *sdph,SdpContext *ctx,SdpPayload *payload)
 	params=&diaparams->video_params;
 	if (params->initialized==0){
 		/* this is the first codec we may accept*/
-		params->localport=payload->localport=lc->rtp_conf.audio_rtp_port;
+		params->localport=payload->localport=lc->rtp_conf.video_rtp_port;
 		params->remoteport=payload->remoteport;
 		params->line=payload->line;
 		params->pt=payload->pt; /* remember the first payload accepted */
@@ -542,7 +553,7 @@ int read_audio_answer(SdpHandler *sdph,SdpContext *ctx,SdpPayload *payload)
 	/* paranoid check: see if this codec is supported in our local rtp profile*/
 	supported=payload_is_supported(payload,lc->local_profile,diaparams->profile);
 	if (!supported) {
-		g_warning("this fucking remote sip phone did not answered properly to my sdp offer!");
+		g_warning("This remote sip phone did not answer properly to my sdp offer!");
 		return 0;
 	}
 	params=&diaparams->audio_params;
@@ -572,13 +583,13 @@ int read_video_answer(SdpHandler *sdph,SdpContext *ctx,SdpPayload *payload)
 	/* paranoid check: see if this codec is supported in our local rtp profile*/
 	supported=payload_is_supported(payload,lc->local_profile,diaparams->profile);
 	if (!supported) {
-		g_warning("this fucking remote sip phone did not answered properly to my sdp offer!");
+		g_warning("This remote sip phone did not answer properly to my sdp offer!");
 		return 0;
 	}
 	params=&diaparams->video_params;
 	if (params->initialized==0){
 		/* this is the first codec we may accept*/
-		params->localport=lc->rtp_conf.audio_rtp_port;
+		params->localport=lc->rtp_conf.video_rtp_port;
 		params->remoteport=payload->remoteport;
 		params->line=payload->line;
 		params->pt=payload->pt; /* remember the first payload accepted */
@@ -587,3 +598,103 @@ int read_video_answer(SdpHandler *sdph,SdpContext *ctx,SdpPayload *payload)
 	}
 }
 
+int subscribe_cb(OsipDialog *dialog, transaction_t *trn, sip_t *msg, void *p2)
+{
+     fprintf(stderr, "%s:%d -- %s\n", __FILE__, __LINE__, __FUNCTION__);
+     return 0;
+}
+
+int subscribe_accepted_cb(OsipDialog *dialog, transaction_t *trn, sip_t *msg, void *p2)
+{
+     fprintf(stderr, "%s:%d -- %s\n", __FILE__, __LINE__, __FUNCTION__);
+     return 0;
+}
+
+int notify_cb(OsipDialog *dialog, transaction_t *trn, sip_t *msg, void *p2)
+{
+     OsipUA *ua = osip_dialog_get_ua(dialog);
+     LinphoneCore *lc = (LinphoneCore*)ua->data;
+     from_t *from= msg->from;
+     header_t *event_header = NULL;
+     body_t *body = NULL;
+     char *bodytext = NULL;
+     char *fromtext = NULL;
+     char lc_message[1024];
+     content_type_t *content_type;
+     char *content_type_text;
+     int rc;
+
+     linphone_core_thread_enter(lc);	
+
+     rc = msg_header_getbyname(msg, "event", 0, &event_header);
+     msg_getbody(msg, 0, &body);
+     if (body)
+	  body_2char(body, &bodytext);
+
+     if (from)
+	  from_2char_without_params(from, &fromtext);
+     sprintf(lc_message, "Received %s notification from %s",
+	     event_header->hvalue, fromtext);
+     fprintf(stderr, "%s\n", lc_message);
+
+     content_type = msg_getcontent_type(msg);
+     rc = content_type_2char(content_type, &content_type_text);
+
+     if (lc->vtable.display_status)
+	  lc->vtable.display_status(lc, lc_message);
+
+     if (lc->vtable.notify_recv) 
+	  lc->vtable.notify_recv(lc, event_header->hvalue, content_type_text, bodytext);
+
+     if (bodytext)
+	  sfree(bodytext);
+     if (fromtext)
+	  sfree(fromtext);
+
+     linphone_core_thread_leave(lc);	
+     
+     return 0;
+}
+
+int message_cb(OsipDialog *dialog, transaction_t *trn, sip_t *msg, void *p2)
+{
+     OsipUA *ua = osip_dialog_get_ua(dialog);
+     LinphoneCore *lc = (LinphoneCore*)ua->data;
+     from_t *from= msg->from;
+     body_t *body = NULL;
+     char *bodytext = NULL;
+     char *fromtext = NULL;
+     char lc_message[1024];
+     content_type_t *content_type;
+     char *content_type_text;
+     int rc;
+
+     linphone_core_thread_enter(lc);	
+
+     msg_getbody(msg, 0, &body);
+     if (body)
+	  body_2char(body, &bodytext);
+
+     if (from)
+	  from_2char_without_params(from, &fromtext);
+     sprintf(lc_message, "Received message from %s", fromtext);
+     fprintf(stderr, "%s\n", lc_message);
+
+     content_type = msg_getcontent_type(msg);
+     rc = content_type_2char(content_type, &content_type_text);
+
+     if (lc->vtable.display_status)
+	  lc->vtable.display_status(lc, lc_message);
+
+     if (lc->vtable.message_recv) 
+	  lc->vtable.message_recv(lc, fromtext, content_type_text, bodytext);
+
+     if (bodytext)
+	  sfree(bodytext);
+     if (fromtext)
+	  sfree(fromtext);
+
+     linphone_core_thread_leave(lc);	
+     
+     return 0;
+}
