@@ -14,6 +14,7 @@
 #include "propertybox.h"
 #include "presence.h"
 #include "addressbook.h"
+#include "friends.h"
 
 typedef struct _LinphoneMainWindow
 {
@@ -23,7 +24,10 @@ typedef struct _LinphoneMainWindow
 	GtkWidget *transferentry;
 	GtkWidget *optioncontrols;
 	GtkWidget *dtmfentry;
+	GtkWidget *buddies;
+	GtkWidget *conference;
 	PresenceBox presencebox;
+	FriendList friendlist;
 }LinphoneMainWindow;
 
 typedef struct _LinphoneGnomeUI
@@ -44,5 +48,4 @@ void linphone_gnome_uninit(LinphoneGnomeUI *ui);
 
 extern LinphoneGnomeUI *uiobj;
 
-
 #endif
