@@ -19,6 +19,9 @@
 #include "linphone.h"
 
 enum{
+#ifdef INET6
+	IFACE_INDEX,
+#endif
 	IFACE_NAME,
 	IFACE_ADDR,
 	IFACE_NCOLUMNS
@@ -26,6 +29,7 @@ enum{
 
 enum {
 	CODEC_NAME,
+	CODEC_RATE,
 	CODEC_BITRATE,
 	CODEC_STATUS,
 	CODEC_PRIVDATA,
@@ -37,10 +41,15 @@ enum {
 #define get_property_box()	(&uiobj->propbox)
 #define property_box_changed() 	gnome_property_box_changed(GNOME_PROPERTY_BOX ( (&uiobj->propbox)->prop))
 
+
+
 void net_section_init(NetSection *sec, GtkWidget *prop)
 {
-	sec->interfaces=lookup_widget(prop, "interfaces");
-	sec->connection_type=lookup_widget(prop,"connection_type");
+	sec->au_port=lookup_widget(prop,"audioport");
+	sec->audio_jittcomp=lookup_widget(prop,"audio_jittcomp");
+#ifdef LINPHONE_DEPRECATED
+	sec->interfaces=lookup_widget(prop, "interfaces");	
+#endif
 	sec->enable_nat=lookup_widget(prop,"enable_nat");
 	sec->nat_address=lookup_widget(prop,"nat_address");
 	sec->nat_label=lookup_widget(prop,"nat_label");
@@ -48,36 +57,38 @@ void net_section_init(NetSection *sec, GtkWidget *prop)
 
 void codec_list_update(GtkTreeView *listview,double bandwidth);
 
-void on_connection_type_selected(NetSection *sec, gint type)
+void on_connection_type_selected(CodecSection *sec, gint type)
 {
-	codec_list_update(GTK_TREE_VIEW(get_property_box()->codec.au_codec_list),bandwidths[type]);
+	/* we must call linphone_core so that codec usability is checked */
+	linphone_core_set_connection_type(get_core(),type);
+	codec_list_update(GTK_TREE_VIEW(sec->au_codec_list),bandwidths[type]);
 	sec->type_sel=type;
 	property_box_changed();
 }
 
 void on_28k_selected(GtkMenuItem *menuitem, gpointer user_data)
 {
-	on_connection_type_selected((NetSection*)user_data , 0);
+	on_connection_type_selected((CodecSection*)user_data , 0);
 }
 
 void on_56k_selected(GtkMenuItem *menuitem, gpointer user_data)
 {
-	on_connection_type_selected((NetSection*)user_data , 1);
+	on_connection_type_selected((CodecSection*)user_data , 1);
 }
 
 void on_128k_selected(GtkMenuItem *menuitem, gpointer user_data)
 {
-	on_connection_type_selected((NetSection*)user_data , 2);
+	on_connection_type_selected((CodecSection*)user_data , 2);
 }
 
 void on_adsl_selected(GtkMenuItem *menuitem, gpointer user_data)
 {
-	on_connection_type_selected((NetSection*)user_data , 3);
+	on_connection_type_selected((CodecSection*)user_data , 3);
 }
 
 void on_ethernet_selected(GtkMenuItem *menuitem, gpointer user_data)
 {
-	on_connection_type_selected((NetSection*)user_data , 4);
+	on_connection_type_selected((CodecSection*)user_data , 4);
 }
 
 void net_selection_changed_cb(GtkTreeSelection *select, gpointer userdata)
@@ -87,12 +98,10 @@ void net_selection_changed_cb(GtkTreeSelection *select, gpointer userdata)
 
 void net_section_fill(NetSection *sec,LinphoneCore *lc)
 {
-	GtkWidget *optionmenu=sec->connection_type;
-	GtkWidget *menu;
-	GtkItem *menu_item;
-	GtkMenuShell *menushell;
+	gfloat value;
+	
 	GList *item_list,*elem;
-	GtkListStore *store = gtk_list_store_new (IFACE_NCOLUMNS, G_TYPE_STRING,G_TYPE_STRING);
+	GtkListStore *store ;
 	GtkTreeIter iter,selected;
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
@@ -100,33 +109,16 @@ void net_section_fill(NetSection *sec,LinphoneCore *lc)
 	gboolean use_nat;
 	gchar *tmp;
 	
-	menu=gtk_option_menu_get_menu(GTK_OPTION_MENU(optionmenu));
-	menushell=GTK_MENU_SHELL(menu);
-	elem=item_list=menushell->children;
 	
-	/* 28k menu item*/
-	menu_item=(GtkItem*)(elem->data);
-	gtk_signal_connect(GTK_OBJECT(menu_item),"activate",(GtkSignalFunc)on_28k_selected,(gpointer)sec);
-	elem=g_list_next(elem);
-	/* 56k menu item */
-    menu_item=(GtkItem*)(elem->data);
-	gtk_signal_connect(GTK_OBJECT(menu_item),"activate",(GtkSignalFunc)on_56k_selected,(gpointer)sec);
-	elem=g_list_next(elem);
-	/* 128k menu item */
-	menu_item=(GtkItem*)(elem->data);
-	gtk_signal_connect(GTK_OBJECT(menu_item),"activate",(GtkSignalFunc)on_128k_selected,(gpointer)sec);
-	elem=g_list_next(elem);
-	/* ADSL menu item */
-	menu_item=(GtkItem*)(elem->data);
-	gtk_signal_connect(GTK_OBJECT(menu_item),"activate",(GtkSignalFunc)on_adsl_selected,(gpointer)sec);
-	elem=g_list_next(elem);
-	/* Ethernet menu item */
-	menu_item=(GtkItem*)(elem->data);
-	gtk_signal_connect(GTK_OBJECT(menu_item),"activate",(GtkSignalFunc)on_ethernet_selected,(gpointer)sec);
-	elem=g_list_next(elem);
-	sec->type_sel=linphone_core_get_connection_type(lc);
-	gtk_option_menu_set_history(GTK_OPTION_MENU(optionmenu),sec->type_sel);
+	value=(gfloat)linphone_core_get_audio_jittcomp(lc);
+	/* put the current value of jitt_comp*/
+	gtk_adjustment_set_value (gtk_range_get_adjustment(GTK_RANGE(sec->audio_jittcomp)),value);
+	/* display current rtp port */
+  	gtk_spin_button_set_value(GTK_SPIN_BUTTON(sec->au_port),
+						(gfloat)linphone_core_get_audio_port(lc));
 	
+#ifdef LINPHONE_DEPRECATED
+	store= gtk_list_store_new (IFACE_NCOLUMNS, G_TYPE_INT, G_TYPE_STRING,G_TYPE_STRING);
 	/* draw the list of interface */
 	elem=linphone_core_get_net_interfaces(lc);
 	while(elem!=NULL){
@@ -136,12 +128,23 @@ void net_section_fill(NetSection *sec,LinphoneCore *lc)
 		if (iface==linphone_core_get_selected_interface(lc)){
 			selected=iter;
 		}
+#ifdef INET6
+		gtk_list_store_set(store,&iter, IFACE_INDEX, iface->index, IFACE_NAME,iface->name,IFACE_ADDR,iface->ipaddr,-1);
+#else
 		gtk_list_store_set(store,&iter,IFACE_NAME,iface->name,IFACE_ADDR,iface->ip4addr,-1);
+#endif
 		elem=g_list_next(elem);
 	}
 	gtk_tree_view_set_model(GTK_TREE_VIEW(sec->interfaces),GTK_TREE_MODEL(store));
 	g_object_unref(G_OBJECT(store));
 	renderer = gtk_cell_renderer_text_new ();
+#ifdef INET6
+	column = gtk_tree_view_column_new_with_attributes (_("Index"),
+                                                   renderer,
+                                                   "text", IFACE_INDEX,
+                                                   NULL);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (sec->interfaces), column);
+#endif
 	column = gtk_tree_view_column_new_with_attributes (_("Name"),
                                                    renderer,
                                                    "text", IFACE_NAME,
@@ -160,6 +163,7 @@ void net_section_fill(NetSection *sec,LinphoneCore *lc)
                   G_CALLBACK (net_selection_changed_cb),
                   NULL);
 	gtk_tree_view_columns_autosize(GTK_TREE_VIEW (sec->interfaces));
+#endif
 	
 	/* nat setup */
 	tmp=linphone_core_get_nat_address(lc,&use_nat);
@@ -202,50 +206,40 @@ void net_section_apply(NetSection *sec,LinphoneCore *lc)
 	GtkTreeSelection *select;
 	GtkTreeIter iter;
 	GtkTreeModel *model;
-	gchar *name;
 	gboolean use_nat;
+	gchar *name;
+#ifdef LINPHONE_DEPRECATED
+#ifdef INET6
+ 	gint index;  	
+#endif
 	/* get network interface*/
 	select = gtk_tree_view_get_selection (GTK_TREE_VIEW (sec->interfaces));
 	if (gtk_tree_selection_get_selected (select, &model, &iter))
 	{
+#ifdef INET6
+ 		gtk_tree_model_get (model, &iter, IFACE_INDEX , &index, -1);
+ 		linphone_core_select_net_interface(lc,index);
+#else
 		gtk_tree_model_get (model, &iter,IFACE_NAME , &name, -1);
 		linphone_core_select_net_interface(lc,name);
 		g_free(name);
+#endif
 	}
 	linphone_core_set_connection_type(lc,sec->type_sel);
+#endif
 	
 	use_nat=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sec->enable_nat));
 	name=gtk_editable_get_chars(GTK_EDITABLE(sec->nat_address),0,-1);
 	linphone_core_set_nat_address(lc,name,use_nat);
-}
-
-void rtp_section_init(RtpSection *sec, GtkWidget *prop)
-{
-	sec->au_port=lookup_widget(prop,"audioport");
-	sec->audio_jittcomp=lookup_widget(prop,"audio_jittcomp");
-}
-
-
-void rtp_section_fill(RtpSection *sec, LinphoneCore *lc)
-{
-	gfloat value;
-	value=(gfloat)linphone_core_get_audio_jittcomp(lc);
-	/* put the current value of jitt_comp*/
-	gtk_adjustment_set_value (gtk_range_get_adjustment(GTK_RANGE(sec->audio_jittcomp)),value);
-	/* display current rtp port */
-  	gtk_spin_button_set_value(GTK_SPIN_BUTTON(sec->au_port),
-						(gfloat)linphone_core_get_audio_port(lc));
-}
-
-
-void rtp_section_apply(RtpSection *sec, LinphoneCore *lc)
-{
+	
+	{
 	/* get the value of jitt_comp*/
 	GtkAdjustment *adj=  gtk_range_get_adjustment(GTK_RANGE(sec->audio_jittcomp));
 	linphone_core_set_audio_jittcomp(lc,(gint)adj->value);
 	/* get  rtp port */
 	adj=gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(sec->au_port));
   	linphone_core_set_audio_port(lc,(gint)adj->value);
+	}
 }
 
 void sip_section_init(SipSection *sec,GtkWidget *prop)
@@ -276,7 +270,7 @@ void sip_section_enable_registrar(SipSection *sec,LinphoneCore *lc, gboolean sta
 	
  	if (state) {
 		gboolean flag;
-		linphone_core_get_registrar_usage(lc,NULL,&flag);
+		linphone_core_get_registrar_usage(lc,NULL,&flag,NULL);
  		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(sec->obproxy_button),flag);
  	}
  	else {
@@ -309,7 +303,7 @@ void sip_section_fill(SipSection *sec, LinphoneCore *lc)
 	tmp=linphone_core_get_addr_of_rec(lc);
 	/* set address of record */
 	if (tmp==NULL) gtk_entry_set_text(GTK_ENTRY(sec->addr_of_rec),tmp);
-	linphone_core_get_registrar_usage(lc,&flag,NULL);
+	linphone_core_get_registrar_usage(lc,&flag,NULL,NULL);
 	if (flag)
 	{
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(sec->registrar_toggle),1);
@@ -357,28 +351,29 @@ void sip_section_apply(SipSection *sec, LinphoneCore *lc)
 		g_free(addr_of_rec);
 		g_free(passwd);
 	}
-	linphone_core_set_registrar_usage(lc,state,as_proxy);
+	linphone_core_set_registrar_usage(lc,state,as_proxy,0);
 }
 
 void codec_section_init(CodecSection *sec, GtkWidget *prop)
 {
 	sec->au_codec_list=lookup_widget(prop,"au_codec_list");
 	sec->codec_info=lookup_widget(prop,"codec_info");
+	sec->connection_type=lookup_widget(prop,"connection_type");
 }
 
 void codec_selection_changed_cb(GtkTreeSelection *selection, gpointer data)
 {
 	GtkTreeIter iter;
     GtkTreeModel *model;
-    MSCodecInfo *codec=NULL;
+    struct _PayloadType *pt=NULL;
 	LinphonePropertyBox *prop=get_property_box();
-
+	
 	if (gtk_tree_selection_get_selected (selection, &model, &iter))	
 	{
-		gtk_tree_model_get (model, &iter, CODEC_PRIVDATA, &codec, -1);
-		g_return_if_fail(codec!=NULL);
+		gtk_tree_model_get (model, &iter, CODEC_PRIVDATA, &pt, -1);
+		g_return_if_fail(pt!=NULL);
 		/* display the codec information */
-		gtk_label_set_text(GTK_LABEL(prop->codec.codec_info),MS_FILTER_INFO(codec)->description);
+		gtk_label_set_text(GTK_LABEL(prop->codec.codec_info),payload_type_get_description(pt));
 	}
 }
 
@@ -386,6 +381,7 @@ void codec_list_build(GtkTreeView *listview,GList *codeclist)
 {
 	GList *elem=codeclist;
 	GtkListStore *store = gtk_list_store_new (CODEC_NCOLUMNS, G_TYPE_STRING,
+																G_TYPE_STRING,
 																G_TYPE_STRING,
 																G_TYPE_STRING,
 																G_TYPE_POINTER,
@@ -396,23 +392,26 @@ void codec_list_build(GtkTreeView *listview,GList *codeclist)
 	GtkTreeSelection *select=gtk_tree_view_get_selection (listview);
 	while(elem!=NULL){
 		gchar *status;
-		gchar *rate;
+		gchar *bitrate,*rate;
 		gchar *color;
-		MSCodecInfo *codec=(MSCodecInfo *)elem->data;
-		if (codec->is_selected) status=_("Enabled");
+		struct _PayloadType *pt=(struct _PayloadType *)elem->data;
+		if (payload_type_enabled(pt)) status=_("Enabled");
 		else status=_("Disabled");
-		if (codec->is_usable) color="blue";
+		if (payload_type_usable(pt)) color="blue";
 		else color="red";
 		/* get an iterator */
 		gtk_list_store_append(store,&iter);
-		rate=g_strdup_printf("%g",((double)codec->bitrate)/1000.0);
-		gtk_list_store_set(store,&iter,	CODEC_NAME,MS_FILTER_INFO(codec)->name,
-										CODEC_BITRATE,rate,
+		bitrate=g_strdup_printf("%g",((double)payload_type_get_bitrate(pt))/1000.0);
+		rate=g_strdup_printf("%i",payload_type_get_rate(pt));
+		gtk_list_store_set(store,&iter,	CODEC_NAME,payload_type_get_mime(pt),
+										CODEC_RATE,rate,
+										CODEC_BITRATE,bitrate,
 										CODEC_STATUS,status,
-										CODEC_PRIVDATA,(gpointer)codec,
+										CODEC_PRIVDATA,(gpointer)pt,
 										CODEC_COLOR,(gpointer)color,
 										-1);
 		g_free(rate);
+		g_free(bitrate);
 		elem=g_list_next(elem);
 	}
 	gtk_tree_view_set_model(listview,GTK_TREE_MODEL(store));
@@ -424,6 +423,12 @@ void codec_list_build(GtkTreeView *listview,GList *codeclist)
 												   "foreground",CODEC_COLOR,
                                                    NULL);
 	gtk_tree_view_append_column (listview, column);
+	column = gtk_tree_view_column_new_with_attributes (_("Rate (Hz)"),
+                                                   renderer,
+                                                   "text", CODEC_RATE,
+												   "foreground",CODEC_COLOR,
+                                                   NULL);
+	gtk_tree_view_append_column (listview, column);
 	column = gtk_tree_view_column_new_with_attributes (_("Status"),
                                                    renderer,
                                                    "text", CODEC_STATUS,
@@ -448,87 +453,202 @@ void codec_list_build(GtkTreeView *listview,GList *codeclist)
 
 void codec_section_fill(CodecSection *sec, LinphoneCore *lc)
 {
+	GtkWidget *optionmenu=sec->connection_type;
+	GtkWidget *menu;
+	GtkItem *menu_item;
+	GtkMenuShell *menushell;
+	GList *item_list,*elem;
 	/* display prefered codecs*/
 	GList *audio_codecs=linphone_core_get_audio_codecs(lc);
 	codec_list_build(GTK_TREE_VIEW(sec->au_codec_list),audio_codecs);
+	g_list_free(audio_codecs);
+	menu=gtk_option_menu_get_menu(GTK_OPTION_MENU(optionmenu));
+	menushell=GTK_MENU_SHELL(menu);
+	elem=item_list=menushell->children;
+	
+	/* 28k menu item*/
+	menu_item=(GtkItem*)(elem->data);
+	gtk_signal_connect(GTK_OBJECT(menu_item),"activate",(GtkSignalFunc)on_28k_selected,(gpointer)sec);
+	elem=g_list_next(elem);
+	/* 56k menu item */
+    menu_item=(GtkItem*)(elem->data);
+	gtk_signal_connect(GTK_OBJECT(menu_item),"activate",(GtkSignalFunc)on_56k_selected,(gpointer)sec);
+	elem=g_list_next(elem);
+	/* 128k menu item */
+	menu_item=(GtkItem*)(elem->data);
+	gtk_signal_connect(GTK_OBJECT(menu_item),"activate",(GtkSignalFunc)on_128k_selected,(gpointer)sec);
+	elem=g_list_next(elem);
+	/* ADSL menu item */
+	menu_item=(GtkItem*)(elem->data);
+	gtk_signal_connect(GTK_OBJECT(menu_item),"activate",(GtkSignalFunc)on_adsl_selected,(gpointer)sec);
+	elem=g_list_next(elem);
+	/* Ethernet menu item */
+	menu_item=(GtkItem*)(elem->data);
+	gtk_signal_connect(GTK_OBJECT(menu_item),"activate",(GtkSignalFunc)on_ethernet_selected,(gpointer)sec);
+	elem=g_list_next(elem);
+	sec->type_sel=linphone_core_get_connection_type(lc);
+	gtk_option_menu_set_history(GTK_OPTION_MENU(optionmenu),sec->type_sel);
+	
+	
 }
 
 void codec_section_apply(CodecSection *sec, LinphoneCore *lc)
 {
 	GtkTreeIter iter;
-	MSCodecInfo *codec;
+	struct _PayloadType *pt;
 	GList *codeclist=NULL;
 	gchar *status;
 	GtkTreeModel *model=gtk_tree_view_get_model(GTK_TREE_VIEW(sec->au_codec_list));
 	/* retrieve the codec list */
 	g_return_if_fail(gtk_tree_model_get_iter_first(model,&iter));
 	do{
-		gtk_tree_model_get (model, &iter, CODEC_STATUS,&status,CODEC_PRIVDATA, &codec,-1);
-		g_return_if_fail(codec!=NULL);
-		if (strcmp(status,_("Enabled"))==0) codec->is_selected=1;
-		else codec->is_selected=0;
-		codeclist=g_list_append(codeclist,(gpointer)codec);
+		gtk_tree_model_get (model, &iter, CODEC_STATUS,&status,CODEC_PRIVDATA, &pt,-1);
+		g_return_if_fail(pt!=NULL);
+		if (strcmp(status,_("Enabled"))==0) payload_type_set_enable(pt,1);
+		else payload_type_set_enable(pt,0);
+		codeclist=g_list_append(codeclist,(gpointer)pt);
 	}while (gtk_tree_model_iter_next(model,&iter));
 	linphone_core_set_audio_codecs(lc,codeclist);
 }
 
 void sound_section_init(SoundSection *sec, GtkWidget *prop)
 {
-	sec->card_combo=lookup_widget(prop,"card_entry");
-	sec->source_combo=lookup_widget(prop,"rec_source");
+	sec->card_entry=lookup_widget(prop,"card_entry");
+	sec->card_combo=lookup_widget(prop,"card_combo");
+	sec->source_entry=lookup_widget(prop,"rec_source");
+	sec->ringfileentry=lookup_widget(prop,"ringfileentry");
 }
 
 void sound_section_fill(SoundSection *sec, LinphoneCore *lc)
 {
-	GtkWidget *child;
 	gchar *dsppath;
 	int devid;
+	SndCardManager *cardmanager;
+	SndCard *card;
+	GList *cardlist=NULL;
+	int i;
 	/* select used sound drivers*/
-  	child=sec->card_combo;
-  	devid=linphone_core_get_sound_device(lc);
-	if (devid==0) {
-		gtk_entry_set_text (GTK_ENTRY (child),"/dev/dsp");
-	}else{
-		dsppath=g_strdup_printf("/dev/dsp%i",devid);
-		gtk_entry_set_text (GTK_ENTRY (child),dsppath);
-		g_free(dsppath);
+
+	devid=linphone_core_get_sound_device(lc);
+	cardmanager=linphone_core_get_card_manager(lc);
+	
+	for (i=0;(card=snd_card_manager_get_card(cardmanager,i))!=0;i++){
+		cardlist=g_list_append(cardlist,snd_card_get_identifier(card));
 	}
-  	/* select audio source*/
-  	child=sec->source_combo;
-  	switch(linphone_core_get_sound_source(lc))
+	gtk_combo_set_popdown_strings(GTK_COMBO(sec->card_combo),cardlist);
+	card=snd_card_manager_get_card(cardmanager,devid);
+	g_return_if_fail(card!=NULL);
+	gtk_entry_set_text (GTK_ENTRY (sec->card_entry),snd_card_get_identifier(card));
+	
+	/* select audio source*/
+#if 1
+	gtk_entry_set_text(GTK_ENTRY (sec->source_entry), linphone_core_get_sound_source(lc));
+#else
+	switch (linphone_core_get_sound_source(lc))
 	{
-		case 'm':
-			gtk_entry_set_text (GTK_ENTRY (child), _("micro"));
+		case "Mic":
+			gtk_entry_set_text (GTK_ENTRY (sec->source_entry), _("Micro"));
   			break;
-  		case 'l':
-  			gtk_entry_set_text (GTK_ENTRY (child), _("line"));
+  		case "Line":
+  			gtk_entry_set_text (GTK_ENTRY (sec->source_entry), _("Line"));
+  			break;
+  		case "Phone":
+  			gtk_entry_set_text (GTK_ENTRY (sec->source_entry), _("Phone"));
+  			break;
+  		case "CD":
+  			gtk_entry_set_text (GTK_ENTRY (sec->source_entry), _("CD"));
+  			break;
+  		case "Video":
+  			gtk_entry_set_text (GTK_ENTRY (sec->source_entry), _("Video"));
+  			break;
+  		case "Aux":
+  			gtk_entry_set_text (GTK_ENTRY (sec->source_entry), _("Aux"));
+  			break;
+  		case "Mix":
+  			gtk_entry_set_text (GTK_ENTRY (sec->source_entry), _("Mix"));
+  			break;
+  		case "Mix Mono":
+  			gtk_entry_set_text (GTK_ENTRY (sec->source_entry), _("Mix Mono"));
   			break;
 		default:
 			g_warning("Invalid source !");
-  	}
+	}
+#endif
+	{
+		gchar *ringfile=linphone_core_get_ring(lc);
+		gnome_file_entry_set_filename(GNOME_FILE_ENTRY(sec->ringfileentry),ringfile);
+		
+  }
+}
+
+
+void
+on_card_changed                        (GtkEditable     *editable,
+                                        gpointer         user_data)
+{
+	gchar *tmp;
+
+	/* get sound device*/
+	tmp=gtk_editable_get_chars(editable,0,-1);
+	linphone_core_set_sound_device_from_name(get_core(),tmp);
+	g_free(tmp);
 }
 
 void sound_section_apply(SoundSection *sec, LinphoneCore *lc)
 {
-	gchar *tmp;
+	gchar *card_name;
+	gchar *source_name;
+	gchar *ring_path;
 	int devid;
-	/* get sound driver mode*/
-	tmp=gtk_editable_get_chars(GTK_EDITABLE(sec->card_combo),0,-1);
-	if (strcmp("/dev/dsp",tmp)==0) devid=0;
-	else devid=atoi(tmp+strlen("/dev/dsp"));
-	linphone_core_set_sound_device(lc,devid);
-	g_free(tmp);
-  	/* get audio source*/
-  	tmp=gtk_editable_get_chars(GTK_EDITABLE(sec->source_combo),0,-1);
-	if (strcmp(tmp,_("micro"))==0) linphone_core_set_sound_source(lc,'m');
-	else if (strcmp(tmp,_("line"))==0) linphone_core_set_sound_source(lc,'l');
+	
+	/* get sound device*/
+	card_name=gtk_editable_get_chars(GTK_EDITABLE(sec->card_entry),0,-1);
+	linphone_core_set_sound_device_from_name(lc,card_name);
+	g_free(card_name);
+	
+	/* get audio source*/
+	source_name=gtk_editable_get_chars(GTK_EDITABLE(sec->source_entry),0,-1);
+#if 1
+	linphone_core_set_sound_source(lc, source_name);
+#else
+	if (strcmp(source_name,_("micro"))==0) linphone_core_set_sound_source(lc,'m');
+	else if (strcmp(source_name,_("line"))==0) linphone_core_set_sound_source(lc,'l');
+#endif
+	g_free(source_name);
+	
+	/* get ring path */
+	ring_path = gtk_editable_get_chars(GTK_EDITABLE(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(sec->ringfileentry))),0,-1);
+	linphone_core_set_ring(lc, ring_path);
+	g_free(ring_path);
+}
+
+void ring_finished(LinphoneCore *lc,gpointer user_data)
+{
+  GtkWidget *button=(GtkWidget*)user_data;
+  LinphonePropertyBox *prop=get_property_box();
+  if (prop->prop==NULL) return; /* the box has been closed before the end of the preview */
+  gtk_widget_set_sensitive(button,1);
+}
+
+void
+on_ringpreview_clicked                 (GtkButton       *button,
+                                        gpointer         user_data)
+{
+  int err;
+  gchar *tmp;
+  LinphonePropertyBox *prop=get_property_box();
+	LinphoneCore *lc=get_core();
+  /* get ring path */
+	tmp=gtk_editable_get_chars(GTK_EDITABLE(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(prop->sound.ringfileentry))),0,-1);
+	err=linphone_core_preview_ring(lc,tmp,ring_finished,(gpointer)button);
+  if (err==0) gtk_widget_set_sensitive(GTK_WIDGET(button),0);
 	g_free(tmp);
 }
 
+
 void linphone_property_box_fill(LinphonePropertyBox * box, LinphoneCore *lp)
 {
 	net_section_fill(&box->net,lp);
-	rtp_section_fill(&box->rtp,lp);
 	sip_section_fill(&box->sip,lp);
 	codec_section_fill(&box->codec,lp);
 	sound_section_fill(&box->sound,lp);
@@ -542,7 +662,6 @@ void linphone_property_box_init(LinphonePropertyBox *box)
 	GtkWidget *prop=create_prop1();
 	box->prop=prop;
 	net_section_init(&box->net,prop);
-	rtp_section_init(&box->rtp,prop);
 	sip_section_init(&box->sip,prop);
 	codec_section_init(&box->codec,prop);
 	sound_section_init(&box->sound,prop);
@@ -564,13 +683,10 @@ void on_propertybox1_apply (GtkButton       *button,
 		net_section_apply(&prop->net,lc);
 		break;
 	case 1:
-		rtp_section_apply(&prop->rtp,lc);
+		sound_section_apply(&prop->sound,lc);
 		break;
 	case 2:
 		sip_section_apply(&prop->sip,lc);
-		break;
-	case 4:
-		sound_section_apply(&prop->sound,lc);
   		break;
   	case 3:
   	 	codec_section_apply(&prop->codec,lc);
@@ -620,8 +736,9 @@ void codec_row_move(GtkTreeView *listview, int action)
     GtkTreeModel *model;
 	GtkTreeSelection *selection;
 	GtkTreePath *treepath;
-    MSCodecInfo *codec=NULL;
+    struct _PayloadType *codec=NULL;
 	gchar *name=NULL;
+	gchar *rate=NULL;
 	gchar *bitrate=NULL;
 	gchar *status=NULL;
 	gchar *color=NULL;
@@ -630,6 +747,7 @@ void codec_row_move(GtkTreeView *listview, int action)
 	if (gtk_tree_selection_get_selected (selection, &model, &iter))	
 	{
 		gtk_tree_model_get (model, &iter, 	CODEC_NAME,&name,
+											CODEC_RATE,&rate,
 											CODEC_BITRATE,&bitrate,
 											CODEC_STATUS,&status,
 											CODEC_PRIVDATA, &codec,
@@ -660,6 +778,7 @@ void codec_row_move(GtkTreeView *listview, int action)
 		}
 		gtk_list_store_set(GTK_LIST_STORE(model),&newiter,
 											CODEC_NAME,name,
+											CODEC_RATE,rate,
 											CODEC_BITRATE,bitrate,
 											CODEC_STATUS,status,
 											CODEC_PRIVDATA, codec,
@@ -670,6 +789,7 @@ void codec_row_move(GtkTreeView *listview, int action)
 		gtk_tree_selection_select_iter(selection,&newiter);
 		g_free(name);
 		g_free(bitrate);
+		g_free(rate);
 		g_free(status);
 		g_free(color);
 	}
@@ -715,14 +835,14 @@ void codec_list_update(GtkTreeView *listview,double bandwidth)
 {
 	GtkTreeModel *model;
 	GtkTreeIter iter;
-	MSCodecInfo *codec;
+	struct _PayloadType *pt;
 	gchar *color;
 	model=gtk_tree_view_get_model(listview);
 	
 	g_return_if_fail(gtk_tree_model_get_iter_first(model,&iter));
 	do{
-		gtk_tree_model_get (model, &iter, 	CODEC_PRIVDATA, &codec,-1);
-		if (ms_codec_is_usable(codec,bandwidth)){
+		gtk_tree_model_get (model, &iter, 	CODEC_PRIVDATA, &pt,-1);
+		if (payload_type_usable(pt)){
 			color="blue";
 		}else color="red";
 		gtk_list_store_set(GTK_LIST_STORE(model),&iter,CODEC_COLOR,color,-1);
@@ -766,12 +886,9 @@ void
 on_user_name_changed                   (GtkEditable     *editable,
                                         gpointer         user_data)
 {
-	GtkWidget *address_of_record=get_property_box()->sip.addr_of_rec;
+	GtkWidget *server=get_property_box()->sip.server;
 	gchar *tmp;
-	/* address of record domain should usually be also the name of the registrar when it is used*/
-	tmp=gtk_editable_get_chars(editable,0,-1);
-	gtk_entry_set_text(GTK_ENTRY(address_of_record),tmp);
-	g_free(tmp);
+	//on_registrar_addr_changed(GTK_EDITABLE(server),NULL);
 	property_box_changed();
 }
 
@@ -810,6 +927,7 @@ on_registrar_addr_changed              (GtkEditable     *editable,
 	hostname=url1->host;
 	addrrec=gtk_editable_get_chars(GTK_EDITABLE(address_of_record),0,-1);
 	url_init(&url2);
+
 	if ((addrrec!=NULL) && (strlen(addrrec)!=0)){
 		err=url_parse(url2,addrrec);
 		//printf("address of record parsed with err=%i\n",err);
@@ -864,12 +982,6 @@ on_address_of_record_changed           (GtkEditable     *editable,
     property_box_changed();
 }
 
-void
-on_card_changed                        (GtkEditable     *editable,
-                                        gpointer         user_data)
-{
-	property_box_changed();
-}
 
 void
 on_audio_jittcomp_value_changed        (GtkRange        *range,
@@ -878,5 +990,24 @@ on_audio_jittcomp_value_changed        (GtkRange        *range,
 	property_box_changed();
 }
 
+void
+on_ringfileentry_changed               (GtkEditable     *editable,
+                                        gpointer         user_data)
+{
+	property_box_changed();
+}
+
+void linphone_property_box_uninit(LinphonePropertyBox *box)
+{
+  memset(box,0, sizeof(LinphonePropertyBox));
+}
 
 
+
+gboolean
+on_property_box_closed                 (GnomeDialog     *gnomedialog,
+                                        gpointer         user_data)
+{
+  linphone_property_box_uninit(get_property_box());
+  return FALSE;
+}
