@@ -18,24 +18,33 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-#ifdef HAVE_ALSA_ASOUNDLIB_H
+#include <config.h>
+
+#if 1 || defined(HAVE_ALSA_ASOUNDLIB_H)
 
 #include "sndcard.h"
+#define ALSA_PCM_NEW_HW_PARAMS_API
 #include <alsa/asoundlib.h>
 struct _AlsaCard
 {
 	SndCard parent;
 	gchar *pcmdev;
-	snd_pcm_t read_handle;
-	snd_pcm_t write_handle;
+	gchar *mixdev;
+	snd_pcm_t *read_handle;
+	snd_pcm_t *write_handle;
 	gint frame_size;
 	gint frames;
-	
+	gchar *readbuf;
+	gint readpos;
+	gchar *writebuf;
+	gint writepos; 
+	snd_mixer_t *mixer;
 };
 
 typedef struct _AlsaCard AlsaCard;
 
 SndCard *alsa_card_new(gint dev_id);
+gint alsa_card_manager_init(SndCardManager *m, gint index);
+
 
 #endif
-
