@@ -32,9 +32,23 @@
 #include "resolver.h"
 #include "digcalc.h"
 
+#ifdef INET6
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <netdb.h>
+#endif
+
 
 list_t callleg_list = { 0, NULL };
 
+
+int osip_ua_has_specific_bind(OsipUA *ua){
+	if (strncmp(ua->contact->url->host,"0.0.0.0",strlen("0.0.0.0"))==0) return 0;
+	if (strncmp(ua->contact->url->host,"[::0]",strlen("[::0]"))==0) return 0;
+	osip_trace(OSIP_INFO1,("ua needs specific binding on %s\n",ua->contact->url->host));
+	return 1;
+}
+
 void
 osip_dialog_init (OsipDialog * new_call, OsipUA * ua)
 {
@@ -135,8 +149,10 @@ osip_dialog_new_from_incoming_trn (transaction_t * trn)
 	OsipUA *ua;
 	sip_t *sipmsg = trn->orig_request;
 	contact_t *ctt = NULL;
+	via_t *via=NULL;
 	char *from = NULL;
 	char *to = NULL;
+	char *expires = NULL;
 	int i;
 
 	call = ua_transaction_get_dialog (trn);
@@ -174,8 +190,11 @@ osip_dialog_new_from_incoming_trn (transaction_t * trn)
 		return NULL;
 	}
 	/* check if the user agent can answer to the call */
-	if (ua->dialog_count >= ua->max_dialogs)
+	if (MSG_IS_INVITE(sipmsg) && ua->dialog_count >= ua->max_dialogs)
 	{
+		osip_trace (OSIP_INFO1,
+			    ("486 busy here: dialog_count=%d >= max_dialogs=%d\n",
+			     ua->dialog_count, ua->max_dialogs));
 		/*respond a 486 busy here */
 		respond_to_request (ua->config, trn, 486);
 		return NULL;
@@ -183,6 +202,7 @@ osip_dialog_new_from_incoming_trn (transaction_t * trn)
 	/* the transaction is accepted, it can open a new dialog */
 
 	call = osip_dialog_alloc (ua);
+	call->sipmethod = sipmsg->strtline->sipmethod;
 
 	if (MSG_IS_INVITE (sipmsg))
 	{
@@ -193,15 +213,15 @@ osip_dialog_new_from_incoming_trn (transaction_t * trn)
 	{
 		osip_dialog_set_state (call, DIALOG_FAKE);
 		ua_transaction_set_incoming_bye_tr (trn, call);
-		/*
-		 * i = osip_dialog_generate_response_default( call, 180,
-		 * trn->orig_request,&sipmsg);
-		 * if (i!=0)
-		 * return NULL;
-		 * osip_dialog_send_response (call, trn, sipmsg);
-		 * 
-		 * What's the goal of this 180 here?????
-		 */
+	}
+	else if (MSG_IS_SUBSCRIBE (sipmsg))
+	{
+		osip_dialog_set_state (call, DIALOG_SUBSCRIBING);
+		ua_transaction_set_incoming_subscribe_tr (trn, call);
+	}
+	else if (MSG_IS_NOTIFY (sipmsg))
+	{
+		ua_transaction_set_incoming_notify_tr (trn, call);
 	}
 	else
 	{
@@ -210,7 +230,31 @@ osip_dialog_new_from_incoming_trn (transaction_t * trn)
 		respond_to_request (ua->config, trn, 501);
 		return NULL;
 	}
-
+	via=list_get(trn->orig_request->vias,0);
+	if (via!=NULL){
+		generic_param_t *received=NULL;
+		via_param_getbyname(via,"received",&received);
+	if (received!=NULL && received->gvalue!=NULL){
+			call->received=sgetcopy(received->gvalue);
+		}else{
+			call->received=sgetcopy(via->host);
+		}
+	}
+	if (osip_ua_has_specific_bind(ua)){
+		call->localip=sgetcopy(ua->contact->url->host);
+	} else {
+	  int i = guess_local_address(call->received,&call->localip);
+	  if (i)
+	    return NULL;
+	}
+	/* if it expires, note that on the dialog */
+	msg_getexpires(sipmsg, 0, &expires);
+	if (expires) {
+		struct timeval current;
+		gettimeofday(&current, NULL);
+		call->expires = atoi(expires);
+		call->expires_time = current.tv_sec + call->expires;
+	}
 	return call;
 }
 
@@ -266,11 +310,14 @@ complete_answer_that_establish_a_dialog (OsipDialog * dialog,
 		list_add (response->record_routes, rr2, -1);
 		pos++;
 	}
-	/*
-	 * contact_clone(ua->contact, &ctt);
-	 * list_add(response->contacts, ctt, 0);
-	 */
-	contact_2char (ua->contact, &p);
+	contact_clone(ua->contact,&ctt);
+	/* modify the contact address so that it match the interface we're going to use
+	to send the response */
+	sfree(ctt->url->host);
+	ctt->url->host=sgetcopy(dialog->localip);
+	contact_2char (ctt, &p);
+	contact_free(ctt);
+	sfree(ctt);
 	msg_setcontact (response, p);
 	sfree (p);
 	/* create a dialog_t if necessary */
@@ -301,7 +348,7 @@ osip_dialog_generate_response_default (OsipDialog * call_leg,
 	if (status < 299 && status > 100 && call_leg == NULL)
 		return -1;	/* not allowed */
 	if (status == 200 &&
-	    (MSG_IS_INVITE (request) || MSG_IS_OPTIONS (request))
+	    (MSG_IS_INVITE (request) || MSG_IS_OPTIONS (request) || MSG_IS_SUBSCRIBE (request))
 	    && call_leg == NULL)
 		return -1;	/* not allowed */
 
@@ -370,7 +417,7 @@ osip_dialog_generate_response_default (OsipDialog * call_leg,
 	if (i != 0)
 		goto grd_error_1;
 
-	if (status < 299 && status > 100 && MSG_IS_INVITE (request))
+	if (status < 299 && status > 100 && (MSG_IS_INVITE (request) || MSG_IS_SUBSCRIBE (request)))
 	{
 		//printf("Completing response******************************\n");
 		complete_answer_that_establish_a_dialog (call_leg, response,
@@ -488,6 +535,7 @@ osip_dialog_register (OsipDialog * call_leg, RegistrationCtxt * context)
 		return (-1);
 	}
 
+	call_leg->sipmethod = "REGISTER";
 	url_init (&rgt);
 	err = url_parse (rgt, context->registrar);
 	if (err != 0)
@@ -553,7 +601,7 @@ osip_dialog_register_with_authentication (OsipDialog * call_leg,
 	int i;
 	RegistrationCtxt *regctx;
 	char *uri;
-	char *pass;
+	char *pass,*p;
 	OsipUA *ua = osip_dialog_get_ua (call_leg);
 
 	if (passwd == NULL)
@@ -578,6 +626,7 @@ osip_dialog_register_with_authentication (OsipDialog * call_leg,
 	if ((wwwauth != NULL) && (proxyauth != NULL))
 		return -1;
 
+	call_leg->sipmethod = "REGISTER";
 	call_leg->reg_context->cseq_number++;
 	i = generating_request_out_of_dialog (call_leg,
 					      "REGISTER", NULL, &sipmesg);
@@ -627,7 +676,7 @@ osip_dialog_register_with_authentication (OsipDialog * call_leg,
 	 */
 	/* !TODO */
 
-	list_add (sipmesg->authorizations, aut, -1);
+	if (aut!=NULL) list_add (sipmesg->authorizations, aut, -1);
 	if (proxy_aut != NULL)
 		list_add (sipmesg->proxy_authorizations, proxy_aut, -1);
 
@@ -680,6 +729,7 @@ osip_dialog_invite (OsipDialog * call_leg, char *callee, char *body_mime)
 	if (call_leg->status != DIALOG_NEW)
 		return (-ESRCH);
 
+	call_leg->sipmethod = "INVITE";
 	/* set to */
 	to_init (&ato);
 	err = to_parse (ato, callee);
@@ -700,6 +750,8 @@ osip_dialog_invite (OsipDialog * call_leg, char *callee, char *body_mime)
 			    ("error: could not build sip message.\n"));
 		return (-1);
 	}
+	printf("adding sdp body...\n");
+	fflush(NULL);
 	/* add the body */
 	if (body_mime != NULL)
 		osip_dialog_add_body_to_request (call_leg, sipmesg,
@@ -741,12 +793,12 @@ osip_dialog_transfer (OsipDialog * call_leg, char *callee, char *body_mime)
 	sfree (ato);
 	if (err == SIP_SYNTAX_ERROR)
 	{
-		osip_trace (OSIP_TRANSFER_INFO, ("Invalid sip address.\n"));
+		osip_trace (OSIP_INFO4, ("Invalid sip address.\n"));
 		return -EINVAL;
 	}
 
 	if (call_leg->status != DIALOG_ESTABLISHED) {
-	    osip_trace(OSIP_TRANSFER_INFO, ("Can not transfer a non-established dialog.\n"));
+	    osip_trace(OSIP_INFO4, ("Can not transfer a non-established dialog.\n"));
 	    return -1;
 	}
 
@@ -766,7 +818,7 @@ osip_dialog_transfer (OsipDialog * call_leg, char *callee, char *body_mime)
 							"INVITE", &sipmesg);
 	if (i != 0)
 	{
-		osip_trace (OSIP_TRANSFER_INFO,
+		osip_trace (OSIP_INFO4,
 			    ("error: could not build sip message.\n"));
 		return (-1);
 	}
@@ -874,6 +926,31 @@ osip_dialog_reinvite_with_authentication (OsipDialog * call_leg,
 	via_free (via);
 	sfree (via);
 
+#ifdef INET6
+	switch (ua->ua_family)
+	{
+	case AF_INET:
+	{
+		char *tmp = (char *) smalloc (90 * sizeof (char));
+		sprintf (tmp, "SIP/2.0/UDP %s:%i;branch=z9hG4bK%u",
+			 ua->ua_ipaddr,
+			 ua->ua_port, via_branch_new_random ());
+		msg_setvia (sipmesg, tmp);
+		sfree (tmp);
+	}
+		break;
+	case AF_INET6:
+	{
+		char *tmp = (char *) smalloc (90 * sizeof (char));
+		sprintf (tmp, "SIP/2.0/UDP [%s]:%i;branch=z9hG4bK%u",
+			 ua->ua_ipaddr,
+			 ua->ua_port, via_branch_new_random ());
+		msg_setvia (sipmesg, tmp);
+		sfree (tmp);
+	}
+		break;
+	}
+#else
 	{
 		char *tmp = (char *) smalloc (90 * sizeof (char));
 		sprintf (tmp, "SIP/2.0/UDP %s:%i;branch=z9hG4bK%u",
@@ -882,7 +959,7 @@ osip_dialog_reinvite_with_authentication (OsipDialog * call_leg,
 		msg_setvia (sipmesg, tmp);
 		sfree (tmp);
 	}
-
+#endif
 	if (sipmesg == NULL)
 	{
 		osip_trace (OSIP_INFO1,
@@ -943,7 +1020,8 @@ osip_dialog_reinvite_with_authentication (OsipDialog * call_leg,
 	/* !TODO */
 
 
-	list_add (sipmesg->authorizations, aut, -1);
+	if (aut!=NULL) 
+		list_add (sipmesg->authorizations, aut, -1);
 	if (proxy_aut != NULL)
 		list_add (sipmesg->proxy_authorizations, proxy_aut, -1);
 	/* !ADD AUTHENTICATION STUFF.... */
@@ -955,6 +1033,237 @@ osip_dialog_reinvite_with_authentication (OsipDialog * call_leg,
 
 }
 
+/**
+ * osip_dialog_subscribe:
+ * @call_leg: a call-leg.
+ * @callee: uri to subscribe to.
+ *
+ * Sends a SUBSCRIBE request to a registrar.
+ *
+ * Returns: 0 if the SUBSCRIBE was sent, a negative value if not.
+ */
+int
+osip_dialog_subscribe (OsipDialog * call_leg, char *callee, char *event_package, char *mimetype)
+{
+	sip_t *sipmesg;
+	char *tmp;
+	from_t *ato;
+	int err;
+	int i;
+
+	OsipUA *ua = osip_dialog_get_ua (call_leg);
+
+	call_leg->sipmethod = "SUBSCRIBE";
+	to_init (&ato);
+	err = to_parse (ato, callee);
+	if (err != 0)
+	{
+		osip_trace (OSIP_INFO1,
+			    ("error: Bad subscription uri (%s).\n",
+			     callee));
+		to_free (ato);
+		sfree (ato);
+		return -EINVAL;
+	}
+
+	i = generating_request_out_of_dialog (call_leg,
+					      "SUBSCRIBE", callee, &sipmesg);
+	/* set Expires header and dialog expires_time */
+	if (call_leg->expires) {
+		struct timeval current;
+		char expires_string[32];
+		gettimeofday(&current, NULL);
+		call_leg->expires_time = current.tv_sec + call_leg->expires;
+		sprintf(expires_string, "%i", call_leg->expires);
+		msg_setexpires(sipmesg, strdup(expires_string));
+	}
+
+	/* set Event header */
+	msg_setheader(sipmesg, "Event", event_package);
+	msg_setaccept(sipmesg, mimetype);
+	call_leg->event_package = event_package;
+	call_leg->mimetype = mimetype;
+	if (i != 0)
+		return (-1);
+	osip_dialog_send_request (call_leg, sipmesg);
+	osip_dialog_set_state (call_leg, DIALOG_SUBSCRIBING);
+	return 0;
+}
+
+/**
+ * osip_dialog_resubscribe:
+ * @call_leg: a call-leg.
+ *
+ * Sends a SUBSCRIBE renewal request.
+ *
+ * Returns: 0 if the SUBSCRIBE was sent, a negative value if not.
+ */
+int
+osip_dialog_resubscribe (OsipDialog * call_leg)
+{
+	char *callee;
+	int rc;
+	url_2char(call_leg->dialog->remote_uri->url, &callee);
+	rc = osip_dialog_subscribe(call_leg, callee, 
+				   call_leg->event_package, call_leg->mimetype);
+	sfree(callee);
+	return rc;
+		
+}
+
+/**
+ * osip_dialog_notify:
+ * @call_leg: a call-leg.
+ * @callee: uri to notify to.
+ * @event_package: type of event
+ * @mimetype: content_type of body
+ * @body: the message body
+ *
+ * Sends a NOTIFY request to a subscriber.
+ *
+ * Returns: 0 if the NOTIFY was sent, a negative value if not.
+ */
+int
+osip_dialog_notify (OsipDialog * call_leg, char *callee, char *event_package, 
+		    char *mimetype, char *body)
+{
+	sip_t *sipmesg;
+	char *tmp;
+	from_t *ato;
+	int err;
+	int i;
+
+	OsipUA *ua = osip_dialog_get_ua (call_leg);
+
+	to_init (&ato);
+	err = to_parse (ato, callee);
+	if (err != 0)
+	{
+		osip_trace (OSIP_INFO1,
+			    ("error: Bad subscription uri (%s).\n",
+			     callee));
+		to_free (ato);
+		sfree (ato);
+		return -EINVAL;
+	}
+
+	i = osip_dialog_generate_request_within_dialog (call_leg,
+							"NOTIFY", &sipmesg);
+	if (i != 0)
+		return (-1);
+	/* set Event header */
+	msg_setheader(sipmesg, "Event", strdup(event_package));
+	if (mimetype) {
+	  content_type_t *content_type;
+	  msg_setcontent_type(sipmesg, mimetype);
+	}
+	if (body)
+	  osip_dialog_add_body_to_request(call_leg, sipmesg, body);
+	osip_dialog_send_request (call_leg, sipmesg);
+	return 0;
+}
+
+/**
+ * osip_dialog_publish:
+ * @call_leg: a call-leg.
+ * @callee: uri to publish to.
+ *
+ * Sends a PUBLISH request to a registrar.
+ *
+ * Returns: 0 if the PUBLISH was sent, a negative value if not.
+ */
+int
+osip_dialog_publish (OsipDialog * call_leg, char *callee, 
+		     char *event_package, char *mimetype, char *body)
+{
+	sip_t *sipmesg;
+	char *tmp;
+	from_t *ato;
+	int err;
+	int i;
+
+	OsipUA *ua = osip_dialog_get_ua (call_leg);
+
+	call_leg->sipmethod = "PUBLISH";
+	to_init (&ato);
+	err = to_parse (ato, callee);
+	if (err != 0)
+	{
+		osip_trace (OSIP_INFO1,
+			    ("error: Bad subscription uri (%s).\n",
+			     callee));
+		to_free (ato);
+		sfree (ato);
+		return -EINVAL;
+	}
+
+	i = generating_request_out_of_dialog (call_leg,
+					      "PUBLISH", callee, &sipmesg);
+	/* set Expires header and dialog expires_time */
+	if (call_leg->expires) {
+		struct timeval current;
+		char expires_string[32];
+		gettimeofday(&current, NULL);
+		call_leg->expires_time = current.tv_sec + call_leg->expires;
+		sprintf(expires_string, "%i", call_leg->expires);
+		msg_setexpires(sipmesg, strdup(expires_string));
+	}
+
+	msg_setcontent_type(sipmesg, mimetype);
+	msg_setbody(sipmesg, body);
+
+	/* set Event header */
+	msg_setheader(sipmesg, "Event", event_package);
+	if (i != 0)
+		return (-1);
+	osip_dialog_send_request (call_leg, sipmesg);
+	osip_dialog_set_state (call_leg, DIALOG_SUBSCRIBING);
+	return 0;
+}
+
+/**
+ * osip_dialog_message:
+ * @call_leg: a call-leg.
+ * @callee: uri to message to.
+ *
+ * Sends a MESSAGE request to callee
+ *
+ * Returns: 0 if the MESSAGE was sent, a negative value if not.
+ */
+int
+osip_dialog_message (OsipDialog * call_leg, char *callee, char *mimetype, char *body)
+{
+	sip_t *sipmesg;
+	char *tmp;
+	from_t *ato;
+	int err;
+	int i;
+
+	OsipUA *ua = osip_dialog_get_ua (call_leg);
+
+	call_leg->sipmethod = "MESSAGE";
+	to_init (&ato);
+	err = to_parse (ato, callee);
+	if (err != 0)
+	{
+		osip_trace (OSIP_INFO1,
+			    ("error: Bad subscription uri (%s).\n",
+			     callee));
+		to_free (ato);
+		sfree (ato);
+		return -EINVAL;
+	}
+
+	i = generating_request_out_of_dialog (call_leg,
+					      "MESSAGE", callee, &sipmesg);
+	msg_setcontent_type(sipmesg, mimetype);
+	msg_setbody(sipmesg, body);
+
+	osip_dialog_send_request (call_leg, sipmesg);
+	osip_dialog_set_state (call_leg, DIALOG_ESTABLISHED);
+	return 0;
+}
+
 sip_t *
 make_cancel (sip_t * request)
 {
@@ -1055,40 +1364,6 @@ osip_dialog_bye (OsipDialog * call_leg)
 }
 
 
-/* Send a notify */
-
-/**
- * osip_dialog_notify:
- * @call_leg: a call-leg.
- *
- *
- * Sends a notify
- *
- * Returns: 0 if the notify was sent, a negative value if not.
- */
-int
-osip_dialog_notify (OsipDialog * call_leg)
-{
-	sip_t *sipmesg = NULL;
-	int i;
-	char *p;
-	transaction_t *trn = call_leg->out_invite_tr;
-	i = osip_dialog_generate_request_within_dialog (call_leg,
-							"NOTIFY",
-							&sipmesg);
-	if (i != 0)
-	    {
-		osip_trace (OSIP_ERROR,
-			    ("Could not generate NOTIFY message.\n"));
-		return (-1);	/* to be set... */
-	    }
-	if (sipmesg == NULL)
-	    return -1;
-	osip_dialog_send_request (call_leg, sipmesg);
-	return (0);
-}
-
-
 /* Send a refer request */
 /**
  * osip_dialog_refer:
@@ -1168,19 +1443,16 @@ dialog_fill_route_set (dialog_t * dialog, sip_t * request)
 	 * with bis-08) then the rquri should contains the remote target
 	 * URI */
 	int i;
-	int pos = 0;
+	int pos = 0, first = 0 ;
 	url_param_t *lr_param;
 	route_t *route;
 	char *last_route;
 	/* AMD bug: fixed 17/06/2002 */
 
+	/* The route_set is stored backwards in CALLER */
 	if (dialog->type == CALLER)
-	{
-		pos = list_size (dialog->route_set) - 1;
-		route = (route_t *) list_get (dialog->route_set, pos);
-	}
-	else
-		route = (route_t *) list_get (dialog->route_set, 0);
+		first = list_size (dialog->route_set) - 1;
+	route = (route_t *) list_get (dialog->route_set, first);
 
 	url_uparam_getbyname (route->url, "lr", &lr_param);
 	if (lr_param != NULL)	/* the remote target URI is the rquri! */
@@ -1230,13 +1502,17 @@ dialog_fill_route_set (dialog_t * dialog, sip_t * request)
 			return -1;
 		if (dialog->type == CALLER)
 		{
-			if (pos != 0)
+			if (pos != first)
 				list_add (request->routes, route2, 0);
+			else
+				route_free(route2);
 		}
 		else
 		{
 			if (!list_eol (dialog->route_set, pos + 1))
 				list_add (request->routes, route2, -1);
+			else
+				route_free(route2);
 		}
 		pos++;
 	}
@@ -1265,6 +1541,7 @@ osip_dialog_generate_request_within_dialog (OsipDialog * call_leg,
 	sip_t *request;
 	char *max_fwd;
 	char *proxy_require;
+	char *localip = call_leg->localip;
 	OsipUA *ua = osip_dialog_get_ua (call_leg);
 
 	i = msg_init (&request);
@@ -1346,6 +1623,45 @@ osip_dialog_generate_request_within_dialog (OsipDialog * call_leg,
 
 	/* even for ACK for 2xx (ACK within a dialog), the branch ID MUST
 	 * be a new ONE! */
+#ifdef INET6
+	{
+		struct addrinfo hints, *res = NULL;
+		char *tmp = (char *) smalloc (90 * sizeof (char));
+		int err;
+
+		memset(&hints, 0, sizeof(hints));
+		hints.ai_family = PF_UNSPEC;
+		hints.ai_socktype = SOCK_DGRAM;
+		hints.ai_flags = AI_NUMERICHOST;
+		err = getaddrinfo(localip, NULL, &hints, &res);
+		if (err) {
+			osip_trace(OSIP_ERROR, ("This should not happen: localip %s is non-numeric\n", localip));
+			return -1;
+		}
+
+		ua->ua_family = res->ai_family;
+		osip_trace(OSIP_INFO1, ("%s: setting ua->ua_family=%d from localip %s\n", 
+					__FUNCTION__, ua->ua_family, localip));
+		switch (ua->ua_family)
+		{
+		case AF_INET:
+			sprintf (tmp, "SIP/2.0/UDP %s:%i;branch=z9hG4bK%u",
+				 call_leg->localip,
+				 ua->ua_port, via_branch_new_random ());
+			break;
+		case AF_INET6:
+			sprintf (tmp, "SIP/2.0/UDP [%s]:%i;branch=z9hG4bK%u",
+				 call_leg->localip,
+				 ua->ua_port, via_branch_new_random ());
+			break;
+		default:
+			break;
+		}
+		msg_setvia (request, tmp);
+		sfree (tmp);
+	}
+
+#else
 	{
 		char *tmp = (char *) smalloc (90 * sizeof (char));
 		sprintf (tmp, "SIP/2.0/UDP %s:%i;branch=z9hG4bK%u",
@@ -1354,12 +1670,14 @@ osip_dialog_generate_request_within_dialog (OsipDialog * call_leg,
 		msg_setvia (request, tmp);
 		sfree (tmp);
 	}
-
+#endif
 	/* add specific headers for each kind of request... */
 	if (0 == strcmp ("INVITE", method_name))
 	{
 		contact_t *ctt;
 		contact_clone (ua->contact, &ctt);
+		sfree(ctt->url->host);
+		ctt->url->host=sgetcopy(call_leg->localip);
 		list_add (request->contacts, ctt, 0);
 	}
 	else if (0 == strcmp ("REFER", method_name)) {
@@ -1440,7 +1758,7 @@ osip_dialog_find (sip_t * sipmsg, OsipDialog ** ret_dialog)
 		{
 			/* this is not yet established dialog */
 			pos++;
-			printf ("not yet established !\n");
+			osip_trace (OSIP_INFO2, ("not yet established !\n"));
 			continue;
 		}
 		/* there are much more things to check: callid, from tags, to etc... */
@@ -1575,6 +1893,11 @@ osip_dialog_ack (OsipDialog * call, transaction_t * trn)
 	int port = 0;
 	int i;
 	route_t *route;
+#ifdef INET6
+	struct sockaddr_storage ss;
+	struct addrinfo hints, *res;
+	int error;
+#endif
 
 	i = osip_dialog_generate_request_within_dialog (call, "ACK", &ack);
 	if (i != 0)
@@ -1608,12 +1931,24 @@ osip_dialog_ack (OsipDialog * call, transaction_t * trn)
 	ua_transaction_get_destination (trn, &dest, &port);
 
 	/* see if we need a name resolution */
+#ifdef INET6
+	memset (&hints, 0, sizeof (hints));
+	hints.ai_family = PF_UNSPEC;
+	hints.ai_socktype = SOCK_DGRAM;
+	hints.ai_flags = AI_NUMERICHOST;
+	error = getaddrinfo (dest, NULL, &hints, &res);
+	if (error)
+#else
 	if (inet_addr (dest) == -1)
+#endif
 	{
 		async_resolv_and_send_ack (ua->manager, call->dialog, dest,
 					   port, ack);
 		return;
 	}
+#ifdef INET6
+	else freeaddrinfo (res);
+#endif
 	/* ack for invite cannot be sent in the same way */
 	udp_send (trn, ack, dest, port, trn->out_socket);
 #ifdef OSIP_RETRANSMISSIONS
@@ -1656,6 +1991,8 @@ generating_request_out_of_dialog (OsipDialog * call_leg,
 	sip_t *request;
 	OsipUA *ua = (OsipUA *) call_leg->ua;
 	RegistrationCtxt *context = call_leg->reg_context;
+	char *localip=NULL;
+
 
 	if (context == NULL && 0 == strcmp ("REGISTER", method_name))
 		return -1;
@@ -1675,15 +2012,7 @@ generating_request_out_of_dialog (OsipDialog * call_leg,
 
 	from_clone (call_leg->from, &(request->from));
 
-	i = from_get_tag (request->from, &tag);
-	if (i == 0)
-	{
-		fprintf (stderr,
-			 "WARNING: call_leg->from already contain a tag??? \n",
-			 callee);
-	}
-	else
-		from_set_tag (request->from, from_tag_new_random ());
+	from_set_tag (request->from, from_tag_new_random ());
 
 	if (0 == strcmp ("REGISTER", method_name))
 	{
@@ -1702,7 +2031,8 @@ generating_request_out_of_dialog (OsipDialog * call_leg,
 				 callee);
 			goto brood_error_1;
 		}
-		if ((ua->registrar != NULL) && (strcmp(ua->registrar->host,request->to->url->host)))
+		if ((ua->registrar != NULL)
+		    && (strcmp (ua->registrar->host, request->to->url->host)))
 		{		/* equal to a pre-existing route set */
 			/* if the pre-existing route set contains a "lr" (compliance
 			 * with bis-08) then the rquri should contains the remote target
@@ -1763,7 +2093,13 @@ generating_request_out_of_dialog (OsipDialog * call_leg,
 				goto brood_error_1;
 		}
 	}
-
+	if (osip_ua_has_specific_bind(ua)){
+		call_leg->localip=sgetcopy(ua->contact->url->host);
+	}else {
+		i=guess_local_address(request->strtline->rquri->host,&call_leg->localip);
+		if (i<0) return -1;
+	}
+	localip=call_leg->localip;
 
 	/* set the cseq and call_id header */
 	if (0 == strcmp ("REGISTER", method_name))
@@ -1779,7 +2115,8 @@ generating_request_out_of_dialog (OsipDialog * call_leg,
 		call_id_setnumber (callid,
 				   sgetcopy (call_leg->reg_context->
 					     callid_number));
-		call_id_sethost (callid, sgetcopy (ua->ua_ip4addr));
+	
+		call_id_sethost (callid, sgetcopy (localip));
 		request->call_id = callid;
 
 		i = cseq_init (&cseq);
@@ -1800,7 +2137,8 @@ generating_request_out_of_dialog (OsipDialog * call_leg,
 		if (i != 0)
 			goto brood_error_1;
 		call_id_setnumber (callid, call_id_new_random ());
-		call_id_sethost (callid, sgetcopy (ua->ua_ip4addr));
+
+		call_id_sethost (callid, sgetcopy (localip));
 		request->call_id = callid;
 
 		i = cseq_init (&cseq);
@@ -1815,31 +2153,67 @@ generating_request_out_of_dialog (OsipDialog * call_leg,
 	/* should be changed to:
 	 * msg_setmax_forwards(request, "10");  */
 
+#ifdef INET6
 	{
+		struct addrinfo hints, *res = NULL;
 		char *tmp = (char *) smalloc (90 * sizeof (char));
-#if 1
-		sprintf (tmp, "SIP/2.0/UDP %s:%i;branch=z9hG4bK%u",
-			 ua->ua_ip4addr,
-			 ua->ua_port, via_branch_new_random ());
-#else
-		sprintf (tmp, "SIP/2.0/UDP %s:%i",
-			 ua->ua_ip4addr,
-			 ua->ua_port);
-#endif
+		int err;
+
+		memset(&hints, 0, sizeof(hints));
+		hints.ai_family = PF_UNSPEC;
+		hints.ai_socktype = SOCK_DGRAM;
+		hints.ai_flags = AI_NUMERICHOST;
+		err = getaddrinfo(localip, NULL, &hints, &res);
+		if (err) {
+			osip_trace(OSIP_ERROR, ("This should not happen: localip %s is non-numeric\n", localip));
+			return -1;
+		}
+
+		ua->ua_family = res->ai_family;
+		osip_trace(OSIP_ERROR, ("%s: setting ua->ua_family=%d from localip %s\n", 
+					__FUNCTION__, ua->ua_family, localip));
+
+		switch (ua->ua_family)
+		{
+		case AF_INET:
+			sprintf (tmp, "SIP/2.0/UDP %s:%i;branch=z9hG4bK%u",
+				 localip,
+				 ua->ua_port, via_branch_new_random ());
+			break;
+		case AF_INET6:
+			sprintf (tmp, "SIP/2.0/UDP [%s]:%i;branch=z9hG4bK%u",
+				 localip,
+				 ua->ua_port, via_branch_new_random ());
+			break;
+		default:
+			break;
+		}
 		msg_setvia (request, tmp);
 		sfree (tmp);
 	}
-
+#else
+	{
+		char *tmp = (char *) smalloc (90 * sizeof (char));
+		sprintf (tmp, "SIP/2.0/UDP %s:%i;branch=z9hG4bK%u",
+			 localip,
+			 ua->ua_port, via_branch_new_random ());
+		msg_setvia (request, tmp);
+		sfree (tmp);
+	}
+#endif
 	/* add specific headers for each kind of request... */
 
-	if (0 == strcmp ("INVITE", method_name))
+	if (   0 == strcmp ("INVITE", method_name)
+	    || 0 == strcmp ("SUBSCRIBE", method_name))
 	{
 		/* add a Contact header for requests that establish a dialog:
-		 * (only "INVITE") */
+		 * (only "INVITE" and "SUBSCRIBE") */
 		/* this Contact is the global location where to send request
 		 * outside of a dialog! like sip:jack@atosc.org? */
 		contact_t *ctt;
 		contact_clone (ua->contact, &ctt);
+		sfree(ctt->url->host);
+		ctt->url->host=sgetcopy(localip);
 		list_add (request->contacts, ctt, 0);
 
 		/* Here we'll add the supported header if it's needed! */
@@ -1850,6 +2224,8 @@ generating_request_out_of_dialog (OsipDialog * call_leg,
 		contact_t *ctt;
 		char exp[20];
 		contact_clone (ua->contact, &ctt);
+		sfree(ctt->url->host);
+		ctt->url->host=sgetcopy(localip);
 		list_add (request->contacts, ctt, 0);
 
 		/* add the expire header == 0 or == expire */
@@ -1930,9 +2306,16 @@ osip_dialog_send_request (OsipDialog * call_leg, sip_t * sipmsg)
 	char *dest;
 	int port = 0;
 	OsipUA *ua = (OsipUA *) call_leg->ua;
+#ifdef INET6
+	struct sockaddr_storage ss;
+	struct addrinfo hints, *res = NULL;
+	int error;
+#endif
 
 	if (MSG_IS_INVITE (sipmsg) || MSG_IS_REGISTER (sipmsg)
-	    || MSG_IS_BYE (sipmsg) || MSG_IS_NOTIFY(sipmsg) || MSG_IS_REFER(sipmsg))
+	    || MSG_IS_SUBSCRIBE (sipmsg) || MSG_IS_NOTIFY (sipmsg)
+	    || MSG_IS_MESSAGE (sipmsg) || MSG_IS_PUBLISH (sipmsg)
+	    || MSG_IS_BYE (sipmsg) || MSG_IS_REFER(sipmsg))
 	{
 		sipevent = osip_new_outgoing_sipmessage (sipmsg);
 		transaction = ua_transaction_new (call_leg, sipmsg);
@@ -1959,10 +2342,23 @@ osip_dialog_send_request (OsipDialog * call_leg, sip_t * sipmsg)
 	/* see if we need a name resolution */
 
 	ua_transaction_get_destination (transaction, &dest, &port);
-	if (inet_addr (dest) == -1)
+#ifdef INET6  
+ 	memset(&hints, 0, sizeof(hints));  
+ 	hints.ai_family = PF_UNSPEC;  
+ 	hints.ai_socktype = SOCK_DGRAM;  
+ 	hints.ai_flags = AI_NUMERICHOST;  
+ 	error = getaddrinfo(dest, NULL, &hints, &res);  
+ 	if (error)  
+#else
+  	if (inet_addr (dest) == -1)
+#endif
+	{
 		return (async_resolv_and_send
 			(ua->manager, transaction, sipevent));
-
+	}
+#ifdef INET6
+	else freeaddrinfo(res);
+#endif
 	ua_transaction_execute (transaction, sipevent);
 	return transaction->transactionid;
 }
@@ -2035,6 +2431,8 @@ osip_dialog_destroy (OsipDialog * call)
 	/* Free the referURL if it is not NULL */
 	if (call->referurl != NULL)
 	    sfree(call->referurl);
+	sfree(call->received);
+	if (call->localip!=NULL) sfree(call->localip);
 	sfree (call);
 	return (err);
 }
@@ -2084,7 +2482,7 @@ osip_dialog_get_body_context (OsipDialog * call, char *body_mime, int pos)
 	{
 		context = (BodyContext *) list_get (&call->body_contexts, i);
 		tmp = body_context_get_mime (context);
-		//printf("Comparing %s <> %s , context=%x, handler=%x\n",tmp,body_mime,context,context->handler);
+		fprintf(stderr, "%s: Comparing %s <> %s , context=%x, handler=%x\n",__FUNCTION__,tmp,body_mime,context,context->handler);
 		if (strcmp (tmp, body_mime) == 0)
 		{
 			if ((body_pos == pos) || (pos == -1))
